@@ -7,9 +7,11 @@
 # This is changed to bash automatically by Makefile for generic Linux
 #
 
-# This is the Click To Continue Theme Specification (ThemeSpec) File with custom placeholders.
+# Title of this theme:
+title="theme_user-email-login-custom-placeholders"
 
 # functions:
+
 download_image_files() {
 	# The list of images to be downloaded is defined in $ndscustomimages ( see near the end of this file )
 	# The source of the images is defined in the openNDS config
@@ -168,14 +170,13 @@ thankyou_page () {
 		</italic-black>
 	"
 
-	if [ -z "$binauth_custom" ]; then
+	binauth_custom="username=$username emailaddress=$emailaddress"
+	encode_custom
+
+	if [ -z "$custom" ]; then
 		customhtml=""
 	else
-		htmlentityencode "$binauth_custom"
-		binauth_custom=$entityencoded
-		# Additionally convert any spaces
-		binauth_custom=$(echo "$binauth_custom" | sed "s/ /\_/g")
-		customhtml="<input type=\"hidden\" name=\"binauth_custom\" value=\"$binauth_custom\">"
+		customhtml="<input type=\"hidden\" name=\"custom\" value=\"$custom\">"
 	fi
 
 	# Continue to the landing page, the client is authenticated there
@@ -499,17 +500,17 @@ additionalthemevars="username emailaddress"
 
 fasvarlist="$fasvarlist $additionalthemevars"
 
-# Title of this theme:
-title="theme_user-email-login-custom-placeholders"
+# You can choose to define a custom string. This will be b64 encoded and sent to openNDS.
+# There it will be made available to be displayed in the output of ndsctl json as well as being sent
+#	to the BinAuth post authentication processing script if enabled.
+# Set the variable $binauth_custom to the desired value.
+# Values set here can be overridden by the themespec file
 
-# You can choose to send a custom data string to BinAuth. Set the variable $binauth_custom to the desired value.
-# Note1: As this script runs on the openNDS router and creates its own log file, there is little point also enabling Binauth.
-#	BinAuth is intended more for use with EXTERNAL FAS servers that don't have direct access to the local router.
-#	Nevertheless it can be enabled at the same time as this script if so desired.
-# Note2: Spaces will be translated to underscore characters.
-# Note3: You must escape any quotes.
 #binauth_custom="This is sample text sent from \"$title\" to \"BinAuth\" for post authentication processing."
 
+# Encode and activate the custom string
+#encode_custom
+
 # Set the user info string for logs (this can contain any useful information)
 userinfo="$title"
 
