@@ -70,16 +70,26 @@ $fullname=$email=$gatewayname=$clientip=$gatewayaddress=$hid=$gatewaymac=$client
 
 $cipher="AES-256-CBC";
 
-//Decrypt and Parse the querystring
-
+// Get the query string components
 if (isset($_GET['status'])) {
-	$redir=$_GET['redir'];
-	$redir_r=explode("fas=", $redir);
-	$fas=$redir_r[1];
-	$iv=$_GET['iv'];
+	@$redir=$_GET['redir'];
+	@$redir_r=explode("fas=", $redir);
+	@$fas=$redir_r[1];
+
+	if (isset($_GET['iv'])) {
+		$iv=$_GET['iv'];
+	} else {
+		$iv="error";
+	}
+
 } else if (isset($_GET['fas']))  {
 	$fas=$_GET['fas'];
-	$iv=$_GET['iv'];
+
+	if (isset($_GET['iv'])) {
+		$iv=$_GET['iv'];
+	} else {
+		$iv="error";
+	}
 } else {
 	exit(0);
 }
@@ -94,7 +104,7 @@ if (isset($_GET['status'])) {
 #
 ####################################################################################################################################
 
-$ndsparamlist=explode(" ", "clientip clientmac gatewayname version hid gatewayaddress gatewaymac authdir originurl clientif admin_email location");
+$ndsparamlist=explode(" ", "clientip clientmac client_type gatewayname version hid gatewayaddress gatewaymac authdir originurl clientif admin_email location");
 
 if (isset($_GET['fas']) and isset($_GET['iv']))  {
 	$string=$_GET['fas'];
@@ -157,11 +167,6 @@ function thankyou_page() {
 	# Be aware that many devices will close the login browser as soon as
 	# the client taps continue, so now is the time to deliver your message.
 
-	# You can also send a custom data string to BinAuth. Set the variable $custom to the desired value
-	# Max length 256 characters
-	$custom="Custom data sent to BinAuth";
-	$custom=base64_encode($custom);
-
 	$me=$_SERVER['SCRIPT_NAME'];
 	$host=$_SERVER['HTTP_HOST'];
 	$fas=$GLOBALS["fas"];
@@ -181,6 +186,16 @@ function thankyou_page() {
 	$redir="http://".$host.$me."?fas=$fas&iv=$iv&landing=1";
 	$tok=hash('sha256', $hid.$key);
 
+	/*	You can also send a custom data string to BinAuth. Set the variable $custom to the desired value
+		It can contain any information that could be used for post authentication processing
+		eg. the values set per client for Time, Data and Data Rate quotas can be sent to BinAuth for a custom script to use
+		This string will be b64 encoded before sending to binauth and will appear in the output of ndsctl json
+	*/
+
+	$custom="fullname=$fullname, email=$email";
+	$custom=base64_encode($custom);
+
+
 	echo "
 		<big-red>
 			Thankyou!
@@ -229,6 +244,7 @@ function write_log() {
 	$user_agent=$_SERVER['HTTP_USER_AGENT'];
 	$clientip=$GLOBALS["clientip"];
 	$clientmac=$GLOBALS["clientmac"];
+	$client_type=$GLOBALS["client_type"];
 	$gatewayname=$GLOBALS["gatewayname"];
 	$gatewayaddress=$GLOBALS["gatewayaddress"];
 	$gatewaymac=$GLOBALS["gatewaymac"];
@@ -240,7 +256,7 @@ function write_log() {
 
 
 	$log=date('Y-m-d H:i:s', $_SERVER['REQUEST_TIME']).
-		", $script, $gatewayname, $fullname, $email, $clientip, $clientmac, $clientif, $user_agent, $redir\n";
+		", $script, $gatewayname, $fullname, $email, $clientip, $clientmac, $client_type, $clientif, $user_agent, $redir\n";
 
 	if ($logpath == "") {
 		$logfile="ndslog/ndslog_log.php";
