@@ -56,7 +56,7 @@ extern pthread_mutex_t config_mutex;
 unsigned int authenticated_since_start = 0;
 
 
-static void binauth_action(t_client *client, const char *reason, char *customdata)
+static void binauth_action(t_client *client, const char *reason, const char *customdata)
 {
 	s_config *config = config_get_config();
 	time_t now = time(NULL);
@@ -70,8 +70,12 @@ static void binauth_action(t_client *client, const char *reason, char *customdat
 	int ret = 1;
 
 	if (config->binauth) {
-		if (!customdata || strlen(customdata) == 0) {
-			customdata="na";
+		debug(LOG_DEBUG, "client->custom=%s", client->custom);
+
+		if (!client->custom || strlen(client->custom) == 0) {
+			customdata="none";
+		} else {
+			customdata=client->custom;
 		}
 
 		uh_urlencode(customdata_enc, sizeof(customdata_enc), customdata, strlen(customdata));
@@ -126,14 +130,12 @@ static void binauth_action(t_client *client, const char *reason, char *customdat
 	}
 }
 
-static int auth_change_state(t_client *client, const unsigned int new_state, const char *reason, char *customdata)
+static int auth_change_state(t_client *client, const unsigned int new_state, const char *reason, const char *customdata)
 {
 	const unsigned int state = client->fw_connection_state;
 	const time_t now = time(NULL);
 	s_config *config = config_get_config();
 
-	debug(LOG_DEBUG, "auth_change_state: customdata [%s]", customdata);
-
 	if (state == new_state) {
 		return -1;
 	} else if (state == FW_MARK_PREAUTHENTICATED) {
@@ -166,6 +168,16 @@ static int auth_change_state(t_client *client, const unsigned int new_state, con
 			client->initial_loop = 1;
 			client->counters.in_window_start = client->counters.incoming;
 			client->counters.out_window_start = client->counters.outgoing;
+
+
+			if (strlen(customdata) > 0) {
+				client->custom = safe_strdup(customdata);
+			} else {
+				client->custom = "bmE=";
+			}
+
+			debug(LOG_DEBUG, "auth_change_state: client->custom=%s ", client->custom);
+
 			binauth_action(client, reason, customdata);
 		} else if (new_state == FW_MARK_BLOCKED) {
 			return -1;
@@ -179,6 +191,7 @@ static int auth_change_state(t_client *client, const unsigned int new_state, con
 			iptables_fw_deauthenticate(client);
 			binauth_action(client, reason, customdata);
 			client_reset(client);
+			client_list_delete(client);
 		} else if (new_state == FW_MARK_BLOCKED) {
 			return -1;
 		} else if (new_state == FW_MARK_TRUSTED) {
@@ -239,7 +252,7 @@ fw_refresh_client_list(void)
 	int routercheck;
 	routercheck = check_routing(watchdog);
 
-	if (config->login_option_enabled == 3 && routercheck == 1) {
+	if (config->login_option_enabled == 3 && routercheck > 0) {
 		/* If the refresh interval has expired, refresh the downloaded remote files.
 			This can be used to update data files or images used by openNDS from storage on a remote server.
 			Access to the openNDS router is not required to update these files as openNDS downloads them.
@@ -389,6 +402,8 @@ fw_refresh_client_list(void)
 				upload_bytes = (cp1->counters.outgoing - cp1->counters.out_window_start);
 				downrate = (download_bytes / 125 / durationsecs); // kbits/sec
 				uprate = (upload_bytes / 125 / durationsecs); // kbits/sec
+				cp1->downrate = downrate;
+				cp1->uprate = uprate;
 
 				debug(LOG_DEBUG, "durationsecs [%llu] download_bytes [%llu] upload_bytes [%llu] ",
 					durationsecs,
@@ -404,10 +419,6 @@ fw_refresh_client_list(void)
 					cp1->upload_rate, uprate
 				);
 
-
-
-				debug(LOG_DEBUG, "rate_exceeded flag is: %u", cp1->rate_exceeded);
-
 				//Handle download rate limiting
 				debug(LOG_DEBUG, "cp1->download_rate: %llu downrate: %llu", cp1->download_rate, downrate);
 
@@ -430,15 +441,20 @@ fw_refresh_client_list(void)
 							uprate
 						);
 						action = ENABLE;
-						iptables_download_ratelimit_enable(cp1, action);
-						//bit 0 is not set so toggle it to signify rate limiting is on
-						cp1->rate_exceeded = cp1->rate_exceeded^1;
+
+						if (config->download_bucket_ratio > 0) {
+							iptables_download_ratelimit_enable(cp1, action);
+							//bit 0 is not set so toggle it to signify rate limiting is on
+							cp1->rate_exceeded = cp1->rate_exceeded^1;
+						} else {
+							debug(LOG_INFO, "Download RATE limiting is disabled");
+						}
 					}
 				}
 
 				if ((cp1->rate_exceeded&1) == 1) {
 					// note checked for bit 0 of rate_exceeded set to 1, it was so we are here 
-					if (cp1->download_rate > 0 && cp1->download_rate >= downrate) {
+					if (cp1->download_rate > 0 && cp1->download_rate > downrate) {
 						debug(LOG_INFO, "Download RATE below quota threshold - bursting allowed: %s %s, in: %llukbits/s, out: %llukbits/s",
 							cp1->ip, cp1->mac,
 							downrate,
@@ -452,9 +468,6 @@ fw_refresh_client_list(void)
 					}
 				}
 
-
-
-
 				//Handle upload rate limiting
 				debug(LOG_DEBUG, "cp1->upload_rate: %llu uprate: %llu", cp1->upload_rate, uprate);
 
@@ -477,9 +490,14 @@ fw_refresh_client_list(void)
 						);
 
 						action = ENABLE;
-						iptables_upload_ratelimit_enable(cp1, action);
-						//bit 1 is not set so toggle it to signify rate limiting is on
-						cp1->rate_exceeded = cp1->rate_exceeded^2;
+
+						if (config->upload_bucket_ratio > 0) {
+							iptables_upload_ratelimit_enable(cp1, action);
+							//bit 1 is not set so toggle it to signify rate limiting is on
+							cp1->rate_exceeded = cp1->rate_exceeded^2;
+						} else {
+							debug(LOG_INFO, "Upload RATE limiting is disabled");
+						}
 					}
 				}
 
@@ -500,6 +518,7 @@ fw_refresh_client_list(void)
 					}
 				}
 
+				debug(LOG_DEBUG, "RATE EXCEEDED flag is %d", cp1->rate_exceeded);
 
 				// start new window
 				cp1->window_start = now;
@@ -615,13 +634,11 @@ end:
  * @return 0 on success
  */
 int
-auth_client_auth_nolock(const unsigned id, const char *reason, char *customdata)
+auth_client_auth_nolock(const unsigned id, const char *reason, const char *customdata)
 {
 	t_client *client;
 	int rc;
 
-	debug(LOG_DEBUG, "authorise client: custom data [%s] ", customdata);
-
 	client = client_list_find_by_id(id);
 
 	// Client should already have hit the server and be on the client list
@@ -639,7 +656,7 @@ auth_client_auth_nolock(const unsigned id, const char *reason, char *customdata)
 }
 
 int
-auth_client_auth(const unsigned id, const char *reason, char *customdata)
+auth_client_auth(const unsigned id, const char *reason, const char *customdata)
 {
 	int rc;
 
