@@ -104,6 +104,8 @@ typedef enum {
 	oRateCheckWindow,
 	oDownloadRate,
 	oUploadRate,
+	oDownloadBucketRatio,
+	oUploadBucketRatio,
 	oDownloadQuota,
 	oUploadQuota,
 	oNdsctlSocket,
@@ -176,6 +178,8 @@ static const struct {
 	{ "ratecheckwindow", oRateCheckWindow },
 	{ "downloadrate", oDownloadRate },
 	{ "uploadrate", oUploadRate },
+	{ "download_bucket_ratio", oDownloadBucketRatio },
+	{ "upload_bucket_ratio", oUploadBucketRatio },
 	{ "downloadquota", oDownloadQuota },
 	{ "uploadquota", oUploadQuota },
 	{ "syslogfacility", oSyslogFacility },
@@ -284,6 +288,8 @@ config_init(void)
 	config.rate_check_window = DEFAULT_RATE_CHECK_WINDOW;
 	config.upload_rate =  DEFAULT_UPLOAD_RATE;
 	config.download_rate = DEFAULT_DOWNLOAD_RATE;
+	config.download_bucket_ratio = DEFAULT_DOWNLOAD_BUCKET_RATIO;
+	config.upload_bucket_ratio =  DEFAULT_UPLOAD_BUCKET_RATIO;
 	config.upload_quota =  DEFAULT_UPLOAD_QUOTA;
 	config.download_quota = DEFAULT_DOWNLOAD_QUOTA;
 	config.syslog_facility = DEFAULT_SYSLOG_FACILITY;
@@ -304,6 +310,7 @@ config_init(void)
 	config.walledgarden_port_list = NULL;
 	config.fas_custom_parameters_list = NULL;
 	config.lockfd = 0;
+	config.online_status = 0;
 
 	// Set up default FirewallRuleSets, and their empty ruleset policies
 	rs = add_ruleset("trusted-users");
@@ -831,6 +838,11 @@ config_read(const char *filename)
 			config.gw_fqdn = safe_strdup(p1);
 			break;
 		case oStatusPath:
+			if (!((stat(p1, &sb) == 0) && S_ISREG(sb.st_mode) && (sb.st_mode & S_IXUSR))) {
+				debug(LOG_ERR, "StatusPath does not exist or is not executeable: %s", p1);
+				debug(LOG_ERR, "Exiting...");
+				exit(1);
+			}
 			config.status_path = safe_strdup(p1);
 			break;
 		case oDhcpDefaultUrlEnable:
@@ -1065,6 +1077,20 @@ config_read(const char *filename)
 				exit(1);
 			}
 			break;
+		case oDownloadBucketRatio:
+			if (sscanf(p1, "%llu", &config.download_bucket_ratio) < 1 || config.download_bucket_ratio < 0) {
+				debug(LOG_ERR, "Bad arg %s to option %s on line %d in %s", p1, s, linenum, filename);
+				debug(LOG_ERR, "Exiting...");
+				exit(1);
+			}
+			break;
+		case oUploadBucketRatio:
+			if (sscanf(p1, "%llu", &config.upload_bucket_ratio) < 1 || config.upload_bucket_ratio < 0) {
+				debug(LOG_ERR, "Bad arg %s to option %s on line %d in %s", p1, s, linenum, filename);
+				debug(LOG_ERR, "Exiting...");
+				exit(1);
+			}
+			break;
 		case oDownloadQuota:
 			if (sscanf(p1, "%llu", &config.download_quota) < 1 || config.download_quota < 0) {
 				debug(LOG_ERR, "Bad arg %s to option %s on line %d in %s", p1, s, linenum, filename);
@@ -1903,7 +1929,7 @@ int set_debuglevel(const char opt[])
 		debug(LOG_DEBUG, "Library command: [%s]", libcmd);
 
 		if (execute_ret_url_encoded(msg, sizeof(msg) - 1, libcmd) == 0) {
-			debug(LOG_NOTICE, "debuglevel [%d] signaled to externals - [%s] acknowledged", level, msg);
+			debug(LOG_DEBUG, "debuglevel [%d] signaled to externals - [%s] acknowledged", level, msg);
 		} else {
 			debug(LOG_ERR, "debuglevel [%d] signaled to externals - unable to set", level);
 		}
