@@ -29,7 +29,7 @@
 #ifndef _CONF_H_
 #define _CONF_H_
 
-#define VERSION "9.4.0"
+#define VERSION "9.5.1"
 
 /*
  * Defines how many times should we try detecting the interface with the default route (in seconds).
@@ -85,6 +85,8 @@
 #define DEFAULT_RATE_CHECK_WINDOW 2 // The data rate check moving average window size multiply this by CHECKINTERVAL to give window size in seconds
 #define DEFAULT_UPLOAD_RATE 0 // 0 means no limit
 #define DEFAULT_DOWNLOAD_RATE 0 // 0 means no limit
+#define DEFAULT_UPLOAD_BUCKET_RATIO 1 // Determines the packet upload queue size per client
+#define DEFAULT_DOWNLOAD_BUCKET_RATIO 1 // Determines the packet download queue size per client
 #define DEFAULT_UPLOAD_QUOTA 0 // 0 means no limit
 #define DEFAULT_DOWNLOAD_QUOTA 0 // 0 means no limit
 #define DEFAULT_LOG_SYSLOG 0
@@ -187,8 +189,7 @@ typedef struct {
 	char *http_encoded_gw_name;			//@brief http encoded name of the gateway, used as a templated variable in splash.htm
 	char *url_encoded_gw_name;			//@brief url encoded name of the gateway used as variable in Preauth
 	char *gw_interface;				//@brief Interface we will manage
-	char *ext_interface;				//@brief Interface of this router which has the routed upstream connection
-	char *ext_gateway;				//@brief The IP address of the upstream gateway
+	char *ext_gateway;				//@brief The interfaces and IP addresses of upstream gateways
 	char *gw_iprange;				//@brief IP range on gw_interface we will manage
 	char *gw_ip;					//@brief Internal IP (v4 or v6) for our web server
 	char *gw_address;				//@brief Internal IP with port for our web server
@@ -230,6 +231,8 @@ typedef struct {
 	int rate_check_window;				//@brief window size in multiples of checkinterval for rate check moving average
 	unsigned long long int download_rate;		//@brief Download rate, kb/s
 	unsigned long long int upload_rate;		//@brief Upload rate, kb/s
+	unsigned long long int download_bucket_ratio;	//@brief used to calculate the packet download queue size per client
+	unsigned long long int upload_bucket_ratio;	//@brief used to calculate the packet upload queue size per client
 	unsigned long long int download_quota;		//@brief Download quota, kB
 	unsigned long long int upload_quota;		//@brief Upload quota, kB
 	int log_syslog;				//@brief boolean, whether to log to syslog
