@@ -264,6 +264,7 @@ setup_from_config(void)
 	char *fashid = NULL;
 	char *phpcmd = NULL;
 	char *preauth_dir = NULL;
+	char *debuglevel = NULL;
 	char libscript[] = "/usr/lib/opennds/libopennds.sh";
 	char gw_name_entityencoded[256] = {0};
 	char gw_name_urlencoded[256] = {0};
@@ -351,6 +352,8 @@ setup_from_config(void)
 	int watchdog = 0;
 	int routercheck;
 
+	// Initialise config->ext_gateway and check router config
+	config->ext_gateway = safe_calloc(SMALL_BUF);
 	routercheck = check_routing(watchdog);
 
 	// Warn if Preemptive Authentication is enabled
@@ -801,13 +804,32 @@ setup_from_config(void)
 	}
 
 	// Preload remote files defined in themespec
-	if (config->login_option_enabled == 3 && routercheck == 1) {
+	if (config->login_option_enabled == 3 && routercheck > 0) {
 		download_remotes(1);
 		// Initial download is in progress, so sleep for a while before starting watchdog thread.
 		sleep(2);
 		config->remotes_last_refresh = now;
 	}
 
+	// Check down/up bucket ratios rates are not less than minimum value of 1
+	if (config->download_bucket_ratio < 1) {
+		debug(LOG_WARNING, "Download bucket ratio setting of %llu disables rate limiting.", config->download_bucket_ratio);
+		config->download_bucket_ratio = 0;
+	}
+
+	if (config->upload_bucket_ratio < 1) {
+		debug(LOG_WARNING, "Upload bucket ratio setting of %llu disables rate limiting.", config->upload_bucket_ratio);
+		config->upload_bucket_ratio = 0;
+	}
+
+
+	// Flag debuglevel to externals
+	safe_asprintf(&debuglevel, "%d", config->debuglevel);
+	if (!set_debuglevel(debuglevel)) {
+		debug(LOG_NOTICE, "Externals flagged with debuglevel %s.", debuglevel);
+	}
+	free(debuglevel);
+
 	debug(LOG_NOTICE, "openNDS is now running.\n");
 }
 
