@@ -170,7 +170,7 @@ usage(void)
 		"Usage: ndsctl [options] command [arguments]\n"
 		"\n"
 		"options:\n"
-		"  -s <path>           Path to the socket\n"
+		"  -s <path>           Deprecated and ignored - Path to the socket, config setting is used instead\n"
 		"  -h                  Print usage\n"
 		"\n"
 		"commands:\n"
@@ -311,7 +311,7 @@ ndsctl_do(const char *socket, const struct argument *arg, const char *param)
 		return 3;
 	}
 
-	if (param) {
+	if (strlen(param) > 0) {
 		snprintf(request, sizeof(request), "%s %s\r\n\r\n", arg->cmd, param);
 	} else {
 		snprintf(request, sizeof(request), "%s\r\n\r\n", arg->cmd);
@@ -413,13 +413,12 @@ main(int argc, char **argv)
 	char argi[128] = {0};
 	char str_b64[QUERYMAXLEN] = {0};
 	char mountpoint[128] = {0};
+	char socket_file[128] = {0};
 	char *cmd;
 	int ret;
 	int lockfd;
 	FILE *fd;
 
-	socket = strdup(DEFAULT_SOCKET_FILENAME);
-
 	// check arguments and take action:
 
 	if (argc <= i) {
@@ -428,8 +427,9 @@ main(int argc, char **argv)
 	}
 
 	if (strcmp(argv[1], "-s") == 0) {
-		if (argc >= 2) {
-			socket = strdup(argv[2]);
+		if (argc >= 4) {
+			// -s option is deprecated - using config setting instead
+			//socket = strdup(argv[2]);
 			i = 3;
 		} else {
 			usage();
@@ -463,6 +463,7 @@ main(int argc, char **argv)
 		return 0;
 	}
 
+
 	// Get the tempfs mountpoint
 	safe_asprintf(&cmd, "/usr/lib/opennds/libopennds.sh tmpfs");
 	fd = popen(cmd, "r");
@@ -487,11 +488,23 @@ main(int argc, char **argv)
 		return ret;
 	} 
 
-	// Get the socket path/filename
-	if (strcmp(socket, DEFAULT_SOCKET_FILENAME) == 0) {
-		free(socket);
-		safe_asprintf(&socket, "%s/%s", mountpoint, DEFAULT_SOCKET_FILENAME);
+	// Get the configured socket filename if there is one
+	safe_asprintf(&cmd, "/usr/lib/opennds/libopennds.sh get_option_from_config ndsctlsocket");
+	fd = popen(cmd, "r");
+	if (fd == NULL) {
+		printf("Unable to open library - Terminating");
+		exit(1);
 	}
+	free(cmd);
+
+	if(fgets(socket_file, sizeof(socket_file), fd) == NULL) {
+		// Not set in config, so using default socket
+		strcat(socket_file, DEFAULT_SOCKET_FILENAME);
+	}
+	pclose(fd);
+
+	// Construct the full socket path/filename
+	safe_asprintf(&socket, "%s/%s", mountpoint, socket_file);
 
 	// check arguments that need socket access and take action:
 	arg = find_argument(argv[i]);
@@ -504,9 +517,9 @@ main(int argc, char **argv)
 	}
 
 	// Collect command line arguments then send the command
-	snprintf(args, sizeof(args), "%s", argv[i+1]);
+	if (argc > i+1) {
+		snprintf(args, sizeof(args), "%s", argv[i+1]);
 
-	if (argc > i) {
 		for (counter=i+2; counter < argc; counter++) {
 			snprintf(argi, sizeof(argi), ",%s", argv[counter]);
 			strncat(args, argi, sizeof(args)-1);
