@@ -85,13 +85,120 @@ extern unsigned int authenticated_since_start;
 extern int created_httpd_threads;
 extern int current_httpd_threads;
 
+int count_substrings(char* string, char* substring) {
+	int idx;
+	int len1;
+	int len2;
+	int numsubs = 0;
+
+	len1 = strlen(string);
+	len2 = strlen(substring);
+
+	for(idx = 0; idx < len1 - len2 + 1; idx++) {
+		if(strstr(string + idx, substring) == string + idx) {
+			numsubs++;
+			idx = idx + len2 -1;
+		}
+	}
+
+	return numsubs;
+}
+
+int startdaemon(char *cmd, int daemonpid)
+{
+	/* Start a program in the background.
+		cmd contains the full path and startup arguments of the program
+		daemonpid returns with the pid of the running program
+		daemonpid returns 0 if the program terminates quickly
+		Function returns 0 if successfully running in background, 1 if failed
+	*/
+
+	char *buff;
+	char *msg;
+	char *daemoncmd;
+	int ret;
+
+	buff = safe_calloc(MID_BUF);
+	msg = safe_calloc(STATUS_BUF);
+
+	b64_encode(buff, MID_BUF, cmd, strlen(cmd));
+
+	safe_asprintf(&daemoncmd, "/usr/lib/opennds/libopennds.sh startdaemon '%s'",
+		buff
+	);
+
+	debug(LOG_DEBUG, "startdaemon command: %s", daemoncmd);
+
+	ret = execute_ret_url_encoded(msg, STATUS_BUF - 1, daemoncmd);
+
+	if (ret == 0) {
+		debug(LOG_DEBUG, "Daemon pid: %s", msg);
+	} else {
+		debug(LOG_INFO, "Failed start daemon from [%s] - retrying", cmd);
+		sleep(1);
+
+		ret = execute_ret_url_encoded(msg, STATUS_BUF - 1, daemoncmd);
+
+		if (ret == 0) {
+			debug(LOG_DEBUG, "Daemon pid: %s", msg);
+		} else {
+			debug(LOG_INFO, "Failed start daemon from [%s] - giving up", cmd);
+		}
+	}
+	free(daemoncmd);
+	free(buff);
+	free(msg);
+	return ret;
+}
+
+int stopdaemon(int daemonpid)
+{
+	/* Stop a program that is running in the background, daemonpid contains the pid of the daemon
+		Returns 0 if successful or 1 if failed to stop.
+		Note: It might fail to stop because it was actually already stopped.
+	*/
+	char *msg;
+	char *daemoncmd;
+	int ret;
+
+	msg = safe_calloc(STATUS_BUF);
+	safe_asprintf(&daemoncmd, "/usr/lib/opennds/libopennds.sh stopdaemon '%d'",
+		daemonpid
+	);
+
+	debug(LOG_DEBUG, "stopdaemon command: %s", daemoncmd);
+
+	ret = execute_ret_url_encoded(msg, STATUS_BUF - 1, daemoncmd);
+
+	if (ret == 0) {
+		debug(LOG_DEBUG, "stopdaemon, pid: [%d], %s", daemonpid, msg);
+	} else {
+		debug(LOG_INFO, "Failed stop daemon pid [%d] - retrying", daemonpid);
+		sleep(1);
+
+		ret = execute_ret_url_encoded(msg, STATUS_BUF - 1, daemoncmd);
+
+		if (ret == 0) {
+			debug(LOG_DEBUG, "stopdaemon, pid: [%d], %s", daemonpid, msg);
+		} else {
+			debug(LOG_INFO, "Failed stop daemon, pid [%d] - giving up", daemonpid);
+		}
+	}
+	free(daemoncmd);
+	free(msg);
+	return ret;
+}
+
 int check_routing(int watchdog)
 {
 	// Check routing configuration
-	char rtest[32] = {0};
+	char *rtest;
 	char *rcmd;
-	const char rtr_fail[] = "-";
-	const char rtr_offline[] = "offline";
+	char rtr_fail[] = "-";
+	char rtr_offline[] = "offline";
+	char rtr_online[] = "online";
+	int online_count;
+	int offline_count;
 	s_config *config = config_get_config();
 
 	safe_asprintf(&rcmd,
@@ -99,27 +206,53 @@ int check_routing(int watchdog)
 		config->gw_interface
 	);
 
-	if (execute_ret_url_encoded(rtest, sizeof(rtest) - 1, rcmd) == 0) {
-		free (rcmd);
+	rtest = safe_calloc(SMALL_BUF);
+
+	if (execute_ret_url_encoded(rtest, SMALL_BUF - 1, rcmd) == 0) {
+		online_count = count_substrings(rtest, rtr_online);
+		offline_count = count_substrings(rtest, rtr_offline);
+
 
 		if (strcmp(rtest, rtr_fail) == 0) {
 			debug(LOG_ERR, "Routing configuration is not valid for openNDS, exiting ...");
 			exit(1);
-		} else if (strcmp(rtest, rtr_offline) == 0) {
-			// offline
-			config->online_status = 0;
-			debug(LOG_WARNING, "Upstream gateway is not connected or is offline");
 		} else {
-			// online
-			if (watchdog == 0 || config->online_status == 0) {
-				config->online_status = 1;
-				config->ext_gateway = strdup(strtok(rtest, " "));
-				config->ext_interface = strdup(strtok(NULL, " "));
-				debug(LOG_NOTICE, "Upstream gateway [ %s ] via interface [ %s ] is online", config->ext_gateway, config->ext_interface);
 
-				//debug(LOG_NOTICE, "Upstream gateway [ address / via interface ] [ %s ] is online", rtest);
+			if (watchdog == 0) {
+				debug(LOG_NOTICE, "Number of Upstream gateway(s) [ %d ]", (online_count + offline_count));
+			}
+
+			if (offline_count > 0) {
+				// An upstream gateway is offline
+				if (online_count == config->online_status) {
+					// no change since last time so issue warning
+					debug(LOG_WARNING, "Upstream gateway(s) [ %s ]", rtest);
+
+				} else if (online_count > config->online_status) {
+					// an interface came online
+					debug(LOG_NOTICE, "Upstream gateway(s) [ %s ]", rtest);
+					config->online_status = online_count;
+
+				} else if (online_count < config->online_status) {
+					// an interface went offline
+					debug(LOG_WARNING, "Upstream gateway(s) [ %s ]", rtest);
+					config->online_status = online_count;
+				}
+			} else {
+
+				if (online_count > config->online_status) {
+					// an interface came online
+					debug(LOG_NOTICE, "Upstream gateway(s) [ %s ]", rtest);
+				}
+
+				config->online_status = online_count;
 			}
 		}
+
+		config->ext_gateway = safe_strdup(rtest);
+		free (rcmd);
+		free (rtest);
+		debug(LOG_DEBUG, "Online Status [ %d ]", config->online_status);
 		return config->online_status;
 	} else {
 		debug(LOG_ERR, "Unable to get routing configuration, exiting ...");
@@ -175,6 +308,7 @@ void ndsctl_unlock()
 int download_remotes(int refresh)
 {
 	char *cmd = NULL;
+	int daemonpid = 0;
 	s_config *config = config_get_config();
 
 	if(refresh == 0) {
@@ -184,7 +318,7 @@ int download_remotes(int refresh)
 	}
 
 	safe_asprintf(&cmd,
-		"/usr/lib/opennds/libopennds.sh download \"%s\" \"%s\" \"%s\" \"%d\" \"%s\" &",
+		"/usr/lib/opennds/libopennds.sh download \"%s\" \"%s\" \"%s\" \"%d\" \"%s\"",
 		config->themespec_path,
 		config->custom_images,
 		config->custom_files,
@@ -192,10 +326,16 @@ int download_remotes(int refresh)
 		config->webroot
 	);
 
-	debug(LOG_DEBUG, "Executing system command: %s\n", cmd);
+	if (config->online_status > 0) {
+		debug(LOG_DEBUG, "Starting daemon: %s\n", cmd);
 
-	if (system(cmd) < 0) {
-		debug(LOG_ERR, "Unable to start remote download - Continuing");
+		if (startdaemon(cmd, daemonpid) == 0) {
+			debug(LOG_DEBUG, "daemon(%s) pid is [%d]", cmd, daemonpid);
+		} else {
+			debug(LOG_DEBUG, "Cannot download remotes - daemon failed to start");
+		}
+	} else {
+		debug(LOG_DEBUG, "Cannot download remotes - upstream gateway(s) are offline");
 	}
 
 	free(cmd);
@@ -375,7 +515,7 @@ char *
 get_iface_ip(const char ifname[], int ip6)
 {
 	char addrbuf[INET6_ADDRSTRLEN] = {0};
-	char cmd[128] = {0};
+	char cmd[256] = {0};
 	char iptype[8] = {0};
 	int i;
 
@@ -525,7 +665,7 @@ int is_addr(const char* addr) {
 void
 ndsctl_status(FILE *fp)
 {
-	char timebuf[32];
+	char timebuf[64];
 	char durationbuf[64];
 	s_config *config;
 	t_client *client;
@@ -533,8 +673,6 @@ ndsctl_status(FILE *fp)
 	unsigned long int now, uptimesecs, durationsecs = 0;
 	unsigned long long int download_bytes, upload_bytes;
 	t_MAC *trust_mac;
-	t_MAC *allow_mac;
-	t_MAC *block_mac;
 	time_t sysuptime;
 	t_WGP *allowed_wgport;
 	t_WGFQDN *allowed_wgfqdn;
@@ -574,10 +712,10 @@ ndsctl_status(FILE *fp)
 	int routercheck;
 	routercheck = check_routing(watchdog);
 
-	if (routercheck == 1) {
-		fprintf(fp, "Upstream gateway [ %s ] via interface [ %s ] is online\n", config->ext_gateway, config->ext_interface);
+	if (routercheck > 0) {
+		fprintf(fp, "Upstream gateway(s) [ %s ]\n", config->ext_gateway);
 	} else {
-		fprintf(fp, "Upstream gateway is offline or not connected\n");
+		fprintf(fp, "All Upstream gateway(s) are offline or not connected [ %s ]\n", config->ext_gateway);
 	}
 
 	fprintf(fp, "MHD Server [ version %s ] listening on: http://%s\n", mhdversion, config->gw_address);
@@ -663,6 +801,13 @@ ndsctl_status(FILE *fp)
 	while (client != NULL) {
 		fprintf(fp, "Client %d\n", indx);
 
+		if (!client->client_type || strlen(client->client_type) == 0) {
+			fprintf(fp, "  Client Type: %s\n", "cpd_can");
+		} else {
+			fprintf(fp, "  Client Type: %s\n", client->client_type);
+		}
+
+
 		fprintf(fp, "  IP: %s MAC: %s\n", client->ip, client->mac);
 
 		format_time(client->counters.last_updated, timebuf);
@@ -739,32 +884,6 @@ ndsctl_status(FILE *fp)
 
 	fprintf(fp, "====\n");
 
-	fprintf(fp, "Blocked MAC addresses:");
-
-	if (config->macmechanism == MAC_ALLOW) {
-		fprintf(fp, " N/A\n");
-	} else  if (config->blockedmaclist != NULL) {
-		fprintf(fp, "\n");
-		for (block_mac = config->blockedmaclist; block_mac != NULL; block_mac = block_mac->next) {
-			fprintf(fp, "  %s\n", block_mac->mac);
-		}
-	} else {
-		fprintf(fp, " none\n");
-	}
-
-	fprintf(fp, "Allowed MAC addresses:");
-
-	if (config->macmechanism == MAC_BLOCK) {
-		fprintf(fp, " N/A\n");
-	} else  if (config->allowedmaclist != NULL) {
-		fprintf(fp, "\n");
-		for (allow_mac = config->allowedmaclist; allow_mac != NULL; allow_mac = allow_mac->next) {
-			fprintf(fp, "  %s\n", allow_mac->mac);
-		}
-	} else {
-		fprintf(fp, " none\n");
-	}
-
 	fprintf(fp, "Trusted MAC addresses:");
 
 	if (config->trustedmaclist != NULL) {
@@ -816,6 +935,13 @@ ndsctl_json_client(FILE *fp, const t_client *client, time_t now, char *indent)
 	fprintf(fp, "  %s\"gatewayaddress\":\"%s\",\n", indent, config->gw_address);
 	fprintf(fp, "  %s\"gatewayfqdn\":\"%s\",\n", indent, config->gw_fqdn);
 	fprintf(fp, "  %s\"version\":\"%s\",\n", indent, VERSION);
+
+	if (!client->client_type || strlen(client->client_type) == 0) {
+		fprintf(fp, "  %s\"client_type\":\"%s\",\n", indent, "cpd_can");
+	} else {
+		fprintf(fp, "  %s\"client_type\":\"%s\",\n", indent, client->client_type);
+	}
+
 	fprintf(fp, "  %s\"mac\":\"%s\",\n", indent, client->mac);
 	fprintf(fp, "  %s\"ip\":\"%s\",\n", indent, client->ip);
 	fprintf(fp, "  %s\"clientif\":\"%s\",\n", indent, clientif);
@@ -831,6 +957,12 @@ ndsctl_json_client(FILE *fp, const t_client *client, time_t now, char *indent)
 	fprintf(fp, "  %s\"token\":\"%s\",\n", indent, client->token ? client->token : "none");
 	fprintf(fp, "  %s\"state\":\"%s\",\n", indent, fw_connection_state_as_string(client->fw_connection_state));
 
+	if (!client->custom || strlen(client->custom) == 0) {
+		fprintf(fp, "  %s\"custom\":\"%s\",\n", indent, "none");
+	} else {
+		fprintf(fp, "  %s\"custom\":\"%s\",\n", indent, client->custom);
+	}
+
 	durationsecs = now - client->session_start;
 	download_bytes = client->counters.incoming;
 	upload_bytes = client->counters.outgoing;
